module CURIC::ScreenScale
  class << self
    attr_accessor :dialog, :disable_ui
  end
  @disable_ui = false

  def self.toggle_dialog
    refresh_workspace

    if @dialog && @dialog.visible?
      @view.remove_observer(@observer) if @observer
      return @dialog.close
    end

    @observer ||= SSViewOB.new
    @view.remove_observer(@observer)
    @view.add_observer(@observer)

    # create new dialog
    d = Sketchup.platform == :platform_win ? 20 : 0
    @dialog = UI::HtmlDialog.new(
      dialog_title: "ScreenScale (#{PLUGIN_VERSION})",
      preferences_key: 'curic.screen_scale.dialog',
      scrollable: true,
      resizable: true,
      width: 250,
      height: 300,
      left: 100,
      top: 100,
      min_width: 230 + d,
      min_height: 70 + d,
      style: UI::HtmlDialog::STYLE_UTILITY
    )

    @dialog.add_action_callback('ready') do |_ac, width, height|
      if Sketchup.platform == :platform_win
        @dialog.set_size(width, height)
        @dialog.execute_script("checkSize(#{[width, height]})")
      end
      refresh_dialog
    end

    @dialog.add_action_callback('fix_size') do |_ac, width, height, old_size|
      @fix_width = old_size.first - width
      @dialog.set_size(width + 2 * @fix_width, height)
    end

    @dialog.add_action_callback('set_screen') do |_ac, data|
      h = data['height'].to_f
      w = data['width'].to_f
      d = data['diagonal'].to_f

      Sketchup.write_default(PLUGIN_NAME, 'resolution_h', h)
      Sketchup.write_default(PLUGIN_NAME, 'resolution_w', w)
      Sketchup.write_default(PLUGIN_NAME, 'diagonal', d)

      pixel = d / Math.sqrt(h * h + w * w)
      Sketchup.write_default(PLUGIN_NAME, 'pixel_size', pixel)

      refresh_dialog
    end
    @dialog.add_action_callback('set_scale') do |_ac, scale|
      @disable_ui = true
      screen_scale(scale)
      @disable_ui = false
    end
    @dialog.add_action_callback('set_size') do |_ac, action, h, w, show_details, visible|
      d = Sketchup.platform == :platform_win ? 20 : 0

      case action
      when 'showDetails'
        if show_details
          @dialog.set_size(w + @fix_width, 170 + d)
        else
          @dialog.set_size(w + @fix_width, 70 + d)
        end
      when 'visibleChange'
        if visible
          @size = [w + @fix_width, h]
          height = 400 + (show_details ? 100 : 0)
          @dialog.set_size(w + @fix_width, height)
        else
          height = 70 + d + (show_details ? 100 : 0)
          @dialog.set_size(w + @fix_width, height)
        end
      when 'focus'
        # @size = [w, h]
        # height = 400 + (show_details ? 100 : 0)
        # @dialog.set_size(w, height)
      when 'blur'
        # UI.start_timer(0.5, false) {
        #   @dialog.set_size(@size[0], @size[1])
        # }
      end
    end

    html = File.join(File.dirname(__FILE__), 'ui/html/screen_scale.html')
    @dialog.set_file(html)
    @dialog.show
  end

  def self.refresh_workspace
    @model = Sketchup.active_model
    @view = @model.active_view
  end

  def self.refresh_dialog
    return unless @dialog && @dialog.visible?
    return if @disable_ui

    screen_data = get_screen_data
    scale = get_current_scale

    perspective = Sketchup.active_model.active_view.camera.perspective?
    screen_data[:perspective] = perspective

    @dialog.execute_script("updateData(#{screen_data.to_json}, '#{scale}')")
  end

  def self.screen_scale(scale = nil)
    m = Sketchup.active_model
    v = m.active_view
    c = v.camera
    vh = v.vpheight
    pixel = Sketchup.read_default(PLUGIN_NAME, 'pixel_size', 0.0104166667)

    vh_size = vh * pixel

    s = Sketchup.read_default(PLUGIN_NAME, 'scale', '1/50')

    unless scale
      r = UI.inputbox(
        ['Scale'],
        [s],
        [SCALES.keys.join('|')],
        'Select Scale'
      )
      return unless r

      Sketchup.write_default(PLUGIN_NAME, 'scale', r[0])

      scale = SCALES[r[0]]
    end

    begin
      c.height = vh_size / scale
    rescue Exception => e
    end
  end

  def self.get_current_scale
    m = Sketchup.active_model
    v = m.active_view
    c = v.camera
    vh = v.vpheight
    pixel = Sketchup.read_default(PLUGIN_NAME, 'pixel_size', 0.0104166667)

    vh_size = vh * pixel

    if c.perspective?
      scale = 'Perspective'
    else
      height = c.height
      true_scale = (height / vh_size).round(1) % 1 == 0
      scale = "#{true_scale ? '' : '~ '}1/#{(height / vh_size).round(0)}"
    end

    scale
  end

  def self.get_screen_data
    data = {}
    h = Sketchup.read_default(PLUGIN_NAME, 'resolution_h')
    w = Sketchup.read_default(PLUGIN_NAME, 'resolution_w')

    scales = []
    SCALES.each do |k, v|
      scales << { key: k, value: v }
    end

    data[:scales] = scales
    data[:height] = h
    data[:width] = w
    data[:diagonal] = Sketchup.read_default(PLUGIN_NAME, 'diagonal', 15.6)
    data[:pixel_size] = Sketchup.read_default(PLUGIN_NAME, 'pixel_size', 0.0104166667)

    data
  end
end
